@@ -29,7 +29,6 @@
 #include "log.h"
 #include "bip.h"
 #include "line.h"
-#include "version.h"
 #include "defaults.h"
 
 int sighup = 0;
@@ -123,7 +122,7 @@ static int add_server(bip_t *bip, struct server *s, list_t *data)
 #define ERRBUFSZ 128
 
 extern list_t *root_list;
-int yyparse();
+extern int yyparse();
 
 void conf_die(bip_t *bip, char *fmt, ...)
 {
@@ -277,9 +276,9 @@ static void usage(char *name)
 static void version()
 {
 	printf(
-"Bip IRC Proxy - %s\n"
+"Bip IRC Proxy - " PACKAGE_VERSION "\n"
 "Copyright © Arnaud Cornet and Loïc Gomez (2004 - 2008)\n"
-"Distributed under the GNU Public License Version 2\n", BIP_VERSION);
+"Distributed under the GNU Public License Version 2\n");
 }
 
 bip_t *_bip;
@@ -1312,7 +1311,7 @@ int main(int argc, char **argv)
 	close(fd);
 
 	bip.listener = listen_new(conf_ip, conf_port, conf_css);
-	if (!bip.listener)
+	if (!bip.listener || bip.listener->connected == CONN_ERROR)
 		fatal("Could not create listening socket");
 
 	for (;;) {
@@ -1571,9 +1570,6 @@ void adm_list_users(struct link_client *ic)
 	hash_iterator_t it;
 	hash_iterator_t lit;
 	char buf[RET_STR_LEN + 1];
-	connection_t *c;
-
-	c = CONN(ic);
 
 	bip_notify(ic, "-- User list");
 	for (hash_it_init(&_bip->users, &it); hash_it_item(&it);
@@ -1621,9 +1617,6 @@ void adm_list_networks(struct link_client *ic)
 {
 	hash_iterator_t it;
 	char buf[RET_STR_LEN + 1];
-	connection_t *c;
-
-	c = CONN(ic);
 
 	bip_notify(ic, "-- Network list (* means SSL):");
 	for (hash_it_init(&_bip->networks, &it); hash_it_item(&it);
@@ -1670,9 +1663,7 @@ noroom:
 void adm_list_connections(struct link_client *ic, struct bipuser *bu)
 {
 	hash_iterator_t it;
-	connection_t *c;
 
-	c = CONN(ic);
 	if (!bu) {
 		bip_notify(ic, "-- Your connections:");
 		bu = LINK(ic)->user;
@@ -1714,8 +1705,10 @@ int ssl_check_trust(struct link_client *ic)
 	int i;
 
 	if(!LINK(ic)->untrusted_certs ||
-			sk_X509_num(LINK(ic)->untrusted_certs) <= 0)
+			sk_X509_num(LINK(ic)->untrusted_certs) <= 0) {
+		ic->allow_trust = 0;
 		return 0;
+    }
 
 	trustcert = sk_X509_value(LINK(ic)->untrusted_certs, 0);
 	strcpy(subject, "Subject: ");
@@ -1753,6 +1746,8 @@ int ssl_check_trust(struct link_client *ic)
 			"Type /QUOTE BIP TRUST OK to trust this "
 			"certificate, /QUOTE BIP TRUST NO to discard it.");
 
+	TYPE(ic) = IRC_TYPE_TRUST_CLIENT;
+	ic->allow_trust = 1;
 	return 1;
 }
 
@@ -1773,18 +1768,10 @@ static int ssl_discard_next_cert(struct link_client *ic)
 int adm_trust(struct link_client *ic, struct line *line)
 {
 	if (ic->allow_trust != 1) {
-		mylog(LOG_ERROR, "User attempted TRUST command without "
-				"being allowed to!");
-		unbind_from_link(ic);
-		return OK_CLOSE;
-	}
-
-	if(!LINK(ic)->untrusted_certs ||
-			sk_X509_num(LINK(ic)->untrusted_certs) <= 0) {
 		/* shouldn't have been asked to /QUOTE BIP TRUST but well... */
 		WRITE_LINE2(CONN(ic), P_SERV, "NOTICE", "TrustEm",
 				"No untrusted certificates.");
-		return ERR_PROTOCOL;
+		return OK_FORGET;
 	}
 
 	if (irc_line_count(line) != 3)
@@ -2268,6 +2255,8 @@ int adm_bip(bip_t *bip, struct link_client *ic, struct line *line, int privmsg)
 			adm_bip_delconn(bip, ic,
 					irc_line_elem(line, privmsg + 2));
 		}
+	} else if (strcasecmp(irc_line_elem(line, privmsg + 1), "TRUST") == 0) {
+		return adm_trust(ic, line);
 	} else {
 		bip_notify(ic, "Unknown command.");
 	}
