@@ -21,8 +21,10 @@ extern int errno;
 static int ssl_initialized = 0;
 static SSL_CTX *sslctx = NULL;
 static int ssl_cx_idx;
+extern FILE *conf_global_log_file;
 static BIO *errbio = NULL;
 extern char *conf_ssl_certfile;
+static int cn_want_write(connection_t *cn);
 static int SSLize(connection_t *cn, int *nc);
 static SSL_CTX *SSL_init_context(void);
 /* SSH like trust management */
@@ -124,6 +126,18 @@ static void connect_trynext(connection_t *cn)
 			continue;
 		}
 
+		if (cn->handle >= FD_SETSIZE) {
+			mylog(LOG_WARN, "too many fd used, close socket %d",
+					cn->handle);
+
+			if (close(cn->handle) == -1)
+				mylog(LOG_WARN, "Error on socket close: %s",
+						strerror(errno));
+
+			cn->handle = -1;
+			break;
+		}
+
 		socket_set_nonblock(cn->handle);
 
 		if (cn->connecting_data->src) {
@@ -326,6 +340,11 @@ static int real_write_all(connection_t *cn)
 	return 0;
 }
 
+/*
+ * May only be used when writing to the client or when sending
+ * timing-sensitive data to the server (PONG, PING for lagtest, QUIT)
+ * because fakelag is not enforced.
+ */
 void write_line_fast(connection_t *cn, char *line)
 {
 	int r;
@@ -353,13 +372,15 @@ void write_line_fast(connection_t *cn, char *line)
 void write_lines(connection_t *cn, list_t *lines)
 {
 	list_append(cn->outgoing, lines);
-	real_write_all(cn);
+	if (cn_want_write(cn))
+		real_write_all(cn);
 }
 
 void write_line(connection_t *cn, char *line)
 {
 	list_add_last(cn->outgoing, bip_strdup(line));
-	real_write_all(cn);
+	if (cn_want_write(cn))
+		real_write_all(cn);
 }
 
 list_t *read_lines(connection_t *cn, int *error)
@@ -718,7 +739,7 @@ static int check_event_write(fd_set *fds, connection_t *cn, int *nc)
 /* token generation interval: 1200ms */
 #define TOKEN_INTERVAL 1200
 
-int cn_want_write(connection_t *cn)
+static int cn_want_write(connection_t *cn)
 {
 	if (cn->anti_flood) {
 		struct timeval tv;
@@ -789,13 +810,8 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 		/*
 		 * This shouldn't happen ! just in case...
 		 */
-		if (cn->handle < 0) {
-			mylog(LOG_WARN, "wait_event invalid socket %d",
-					cn->handle);
-			if (cn_is_connected(cn))
-				cn->connected = CONN_ERROR;
-			continue;
-		}
+		if (cn->handle < 0 || cn->handle >= FD_SETSIZE)
+			fatal("wait_event invalid socket %d", cn->handle);
 
 		/* exceptions are OOB and disconnections */
 		FD_SET(cn->handle, &fds_except);
@@ -966,6 +982,18 @@ static void create_listening_socket(char *hostname, char *port,
 			continue;
 		}
 
+		if (cn->handle >= FD_SETSIZE) {
+			mylog(LOG_WARN, "too many fd used, close listening socket %d",
+					cn->handle);
+
+			if (close(cn->handle) == -1)
+				mylog(LOG_WARN, "Error on socket close: %s",
+						strerror(errno));
+
+			cn->handle = -1;
+			break;
+		}
+
 		if (setsockopt(cn->handle, SOL_SOCKET, SO_REUSEADDR,
 					(char *)&multi_client,
 					sizeof(multi_client)) < 0) {
@@ -1090,7 +1118,7 @@ static DH *dh_1024(void)
 }
 
 /* ripped from postfix's tls_dh.c */
-static DH *tmp_dh_cb(SSL *ssl_unused, int export, int keylength)
+static DH *tmp_dh_cb(UNUSED(SSL *ssl_unused), int export, int keylength)
 {
 	DH *ret;
 
@@ -1113,10 +1141,21 @@ connection_t *accept_new(connection_t *cn)
 
 	mylog(LOG_DEBUG, "Trying to accept new client on %d", cn->handle);
 	err = accept(cn->handle, &sa, &sa_len);
+
 	if (err < 0) {
-		mylog(LOG_ERROR, "accept failed: %s", strerror(errno));
+		fatal("accept failed: %s", strerror(errno));
+	}
+
+	if (err >= FD_SETSIZE) {
+		mylog(LOG_WARN, "too many client connected, close %d", err);
+
+		if (close(err) == -1)
+			mylog(LOG_WARN, "Error on socket close: %s",
+					strerror(errno));
+
 		return NULL;
 	}
+
 	socket_set_nonblock(err);
 
 	conn = connection_init(cn->anti_flood, cn->ssl, cn->timeout, 0);
@@ -1202,7 +1241,7 @@ static SSL_CTX *SSL_init_context(void)
 	if (!ssl_initialized) {
 		SSL_library_init();
 		SSL_load_error_strings();
-		errbio = BIO_new_fp(stderr,BIO_NOCLOSE);
+		errbio = BIO_new_fp(conf_global_log_file, BIO_NOCLOSE);
 
 		ssl_cx_idx = SSL_get_ex_new_index(0, "bip connection_t",
 			NULL, NULL,NULL);
@@ -1351,7 +1390,7 @@ static int SSLize(connection_t *cn, int *nc)
 	ERR_print_errors(errbio);
 
 	if (err2 == SSL_ERROR_NONE) {
-		SSL_CIPHER *cipher;
+		const SSL_CIPHER *cipher;
 		char buf[128];
 		int len;
 
@@ -1397,6 +1436,7 @@ static int SSLize(connection_t *cn, int *nc)
 	/* From now on, we are on error, thus we return 1 to check timeout */
 	if (err2 == SSL_ERROR_ZERO_RETURN || err2 == SSL_ERROR_SSL) {
 		mylog(LOG_ERROR, "Error in SSL handshake.");
+		connection_close(cn);
 		cn->connected = CONN_ERROR;
 		return 1;
 	}
