@@ -22,7 +22,6 @@
 #include "log.h"
 #include "connection.h"
 #include "md5.h"
-#include "version.h"
 
 #define S_CONN_DELAY (10)
 
@@ -309,7 +308,7 @@ static int irc_352(struct link_server *server, struct line *line)
 	return OK_COPY_WHO;
 }
 
-static int irc_315(struct link_server *server, struct line *l)
+static int irc_315(struct link_server *server, UNUSED(struct line *l))
 {
 	struct link *link = LINK(server);
 	if (link->who_client) {
@@ -326,7 +325,6 @@ static int irc_315(struct link_server *server, struct line *l)
 				link->who_client, link->who_client->who_count);
 		}
 	}
-	l = NULL; /* keep gcc happy */
 
 	return OK_COPY_WHO;
 }
@@ -767,8 +765,6 @@ static int irc_cli_startup(bip_t *bip, struct link_client *ic,
 	if (LINK(ic)->s_state != IRCS_CONNECTED) {
 		/* Check if we have an untrusted certificate from the server */
 		if (ssl_check_trust(ic)) {
-			TYPE(ic) = IRC_TYPE_TRUST_CLIENT;
-			ic->allow_trust = 1;
 			free(init_nick);
 			return OK_FORGET;
 		}
@@ -1458,7 +1454,7 @@ static int irc_367(struct link_server *server, struct line *l)
 }
 
 /* same as irc_315 */
-static int irc_368(struct link_server *server, struct line *l)
+static int irc_368(struct link_server *server, UNUSED(struct line *l))
 {
 	struct link *link = LINK(server);
 	if (link->who_client) {
@@ -1476,7 +1472,6 @@ static int irc_368(struct link_server *server, struct line *l)
 				link->who_client, link->who_client->who_count);
 		}
 	}
-	l = NULL; /* keep gcc happy */
 
 	return OK_COPY_WHO;
 }
@@ -1838,7 +1833,7 @@ static void irc_privmsg_check_ctcp(struct link_server *server,
 	nick = nick_from_ircmask(line->origin);
 	if (irc_line_elem_equals(line, 2, "\001VERSION\001")) {
 		WRITE_LINE2(CONN(server), NULL, "NOTICE", nick,
-				"\001VERSION bip-" BIP_VERSION "\001");
+				"\001VERSION bip-" PACKAGE_VERSION "\001");
 	}
 	free(nick);
 }
@@ -2444,9 +2439,10 @@ void bip_on_event(bip_t *bip, connection_t *conn)
 
 	if (conn == bip->listener) {
 		struct link_client *n = irc_accept_new(conn);
-		assert(n);
-		list_add_last(&bip->conn_list, CONN(n));
-		list_add_last(&bip->connecting_client_list, n);
+		if (n) {
+			list_add_last(&bip->conn_list, CONN(n));
+			list_add_last(&bip->connecting_client_list, n);
+		}
 		return;
 	}
 
